from django.db import models


class BaseData(models.Model):
    namespace = models.CharField(max_length=255, blank=True, null=False)

    class Meta:
        abstract = True


class BotData(BaseData):
    data = models.JSONField()


class ChatData(BaseData):
    chat_id = models.BigIntegerField(null=False, blank=False)
    data = models.JSONField()

    class Meta:
        constraints = [models.UniqueConstraint(fields=["namespace", "chat_id"], name="unique_chat_id")]
        indexes = [models.Index(fields=["namespace", "chat_id"])]


class UserData(BaseData):
    user_id = models.BigIntegerField(unique=True, null=False, blank=False)
    data = models.JSONField()

    class Meta:
        constraints = [models.UniqueConstraint(fields=["namespace", "user_id"], name="unique_user_id")]
        indexes = [models.Index(fields=["namespace", "user_id"])]


class CallbackData(BaseData):
    data = models.JSONField()


class ConversationData(BaseData):
    name = models.CharField(max_length=255, blank=True, null=False)
    key = models.TextField()  # stringified JSON here!
    state = models.JSONField(null=True, blank=True)

    class Meta:
        constraints = [models.UniqueConstraint(fields=["namespace", "name", "key"], name="unique_conversation_data")]
        indexes = [models.Index(fields=["namespace", "name", "key"])]



# Generated by Django 3.2.5 on 2021-07-19 19:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('python_telegram_bot_django_persistence', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='conversationdata',
            name='state',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



from django.apps import AppConfig


class PythonTelegramBotDjangoPersistenceConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "python_telegram_bot_django_persistence"



from django.contrib import admin

from .models import (
    BotData,
    CallbackData,
    ChatData,
    ConversationData,
    UserData,
)

admin.site.register(BotData)
admin.site.register(ChatData)
admin.site.register(UserData)
admin.site.register(CallbackData)
admin.site.register(ConversationData)



import json
from collections import defaultdict
from typing import DefaultDict, Optional, Tuple, cast

from telegram.ext import BasePersistence
from telegram.ext.utils.types import BD, CD, UD, CDCData, ConversationDict

from .models import (
    BotData,
    CallbackData,
    ChatData,
    ConversationData,
    UserData,
)


class DjangoPersistence(BasePersistence[UD, CD, BD]):
    def __init__(
        self,
        namespace: str = "",
        store_user_data: bool = True,
        store_chat_data: bool = True,
        store_bot_data: bool = True,
        store_callback_data: bool = False,
    ):
        super().__init__(
            store_user_data=store_user_data,
            store_chat_data=store_chat_data,
            store_bot_data=store_bot_data,
            store_callback_data=store_callback_data,
        )
        self._namespace = namespace

    def get_bot_data(self) -> BD:
        try:
            return BotData.objects.get(namespace=self._namespace).data
        except BotData.DoesNotExist:
            return {}

    def update_bot_data(self, data: BD) -> None:
        BotData.objects.update_or_create(namespace=self._namespace, defaults={"data": data})

    def refresh_bot_data(self, bot_data: BD) -> None:
        if isinstance(bot_data, dict):
            orig_keys = set(bot_data.keys())
            bot_data.update(self.get_bot_data())
            for key in orig_keys - set(bot_data.keys()):
                bot_data.pop(key)

    def get_chat_data(self) -> DefaultDict[int, CD]:
        return defaultdict(
            dict, {data.chat_id: data.data for data in ChatData.objects.filter(namespace=self._namespace)}
        )

    def update_chat_data(self, chat_id: int, data: CD) -> None:
        ChatData.objects.update_or_create(namespace=self._namespace, chat_id=chat_id, defaults={"data": data})

    def refresh_chat_data(self, chat_id: int, chat_data: CD) -> None:
        try:
            if isinstance(chat_data, dict):
                orig_keys = set(chat_data.keys())
                chat_data.update(ChatData.objects.get(namespace=self._namespace, chat_id=chat_id).data)
                for key in orig_keys - set(chat_data.keys()):
                    chat_data.pop(key)
        except ChatData.DoesNotExist:
            pass

    def get_user_data(self) -> DefaultDict[int, UD]:
        return defaultdict(
            dict, {data.user_id: data.data for data in UserData.objects.filter(namespace=self._namespace)}
        )

    def update_user_data(self, user_id: int, data: UD) -> None:
        UserData.objects.update_or_create(namespace=self._namespace, user_id=user_id, defaults={"data": data})

    def refresh_user_data(self, user_id: int, user_data: UD) -> None:
        try:
            if isinstance(user_data, dict):
                orig_keys = set(user_data.keys())
                user_data.update(UserData.objects.get(namespace=self._namespace, user_id=user_id).data)
                for key in orig_keys - set(user_data.keys()):
                    user_data.pop(key)
        except UserData.DoesNotExist:
            pass

    def get_callback_data(self) -> Optional[CDCData]:
        try:
            cdcdata_json = CallbackData.objects.get(namespace=self._namespace)
            # Before asking me wtf is this, just check DictPersistence
            return cast(CDCData, ([(one, float(two), three) for one, two, three in cdcdata_json[0]], cdcdata_json[1]))
        except CallbackData.DoesNotExist:
            return None

    def update_callback_data(self, data: CDCData) -> None:
        CallbackData.objects.update_or_create(namespace=self._namespace, defaults=data)

    def get_conversations(self, name: str) -> ConversationDict:
        return {
            tuple(json.loads(data.key)): data.state
            for data in ConversationData.objects.filter(namespace=self._namespace, name=name)
        }

    def update_conversation(self, name: str, key: Tuple[int, ...], new_state: Optional[object]) -> None:
        ConversationData.objects.update_or_create(
            namespace=self._namespace,
            name=name,
            key=json.dumps(key, sort_keys=True),
            defaults={"state": new_state},
        )

    def flush(self) -> None:
        pass



# Generated by Django 3.2.5 on 2021-07-19 17:54

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='BotData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namespace', models.CharField(blank=True, max_length=255)),
                ('data', models.JSONField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CallbackData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namespace', models.CharField(blank=True, max_length=255)),
                ('data', models.JSONField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ChatData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namespace', models.CharField(blank=True, max_length=255)),
                ('chat_id', models.BigIntegerField()),
                ('data', models.JSONField()),
            ],
        ),
        migrations.CreateModel(
            name='ConversationData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namespace', models.CharField(blank=True, max_length=255)),
                ('name', models.CharField(blank=True, max_length=255)),
                ('key', models.TextField()),
                ('state', models.JSONField()),
            ],
        ),
        migrations.CreateModel(
            name='UserData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namespace', models.CharField(blank=True, max_length=255)),
                ('user_id', models.BigIntegerField(unique=True)),
                ('data', models.JSONField()),
            ],
        ),
        migrations.AddIndex(
            model_name='userdata',
            index=models.Index(fields=['namespace', 'user_id'], name='python_tele_namespa_3c44b8_idx'),
        ),
        migrations.AddConstraint(
            model_name='userdata',
            constraint=models.UniqueConstraint(fields=('namespace', 'user_id'), name='unique_user_id'),
        ),
        migrations.AddIndex(
            model_name='conversationdata',
            index=models.Index(fields=['namespace', 'name', 'key'], name='python_tele_namespa_1b4da9_idx'),
        ),
        migrations.AddConstraint(
            model_name='conversationdata',
            constraint=models.UniqueConstraint(fields=('namespace', 'name', 'key'), name='unique_conversation_data'),
        ),
        migrations.AddIndex(
            model_name='chatdata',
            index=models.Index(fields=['namespace', 'chat_id'], name='python_tele_namespa_31f032_idx'),
        ),
        migrations.AddConstraint(
            model_name='chatdata',
            constraint=models.UniqueConstraint(fields=('namespace', 'chat_id'), name='unique_chat_id'),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['python_telegram_bot_django_persistence',
 'python_telegram_bot_django_persistence.migrations']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.1', 'python-telegram-bot>=13.7']

setup_kwargs = {
    'name': 'python-telegram-bot-django-persistence',
    'version': '0.1.3',
    'description': 'Package to use Django ORM as persistence engine in Python Telegram Bot',
    'long_description': None,
    'author': 'Alexander Shishenko',
    'author_email': 'alex@shishenko.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
